"""
Streamlit Application
Nicolas Atrax
Marie FU
"""

import os
import shutil
import tempfile
from zipfile import ZipFile
import streamlit as st
import csv
import re
import codecs
import src.basic as tmp
from src.gerneralFileOP import *
from src.csvFileOP import *


# Define constant for GarganText TSV columns
TSV_COLUMNS = ["Publication Day", "Publication Month", "Publication Year", "Authors", "Title", "Source", "Abstract"]
TSV_DATE_COLUMNS = ["Publication Day", "Publication Month", "Publication Year"]

STATE = st.session_state

tmp.base("CleanCSVtoTSV")


def checkDate(name : str, value : str, csvLine : int) -> None:

    """
    Check if the date value is in a correct format

    Args:
        name (String) : Column name
        value (String) : Value of cell
        csvLine (Int) : Current line

    Returns:
        None

    Authors:
        Nicolas Atrax
        Marie FU
    """
    
    if value == "" or value == "\n":
        STATE.fileData += "1\t"
    elif not re.match(r'^[0-9]+$',value) or re.match(r'^0$',value):
        STATE.errMessageLog += f"{STATE.general_text_dict['err_date_value1']} {csvLine}, {STATE.general_text_dict['err_date_value2']} \"{value}\". \"{name}\" {STATE.general_text_dict['err_date_value3']} \n\n"
        STATE.contentProblem = True
    else:
        STATE.fileData += f"{value}\t"

    return


def checkMissing(name : str, value : str, csvLine : int) -> None:

    """
    Check value

    Args:
        name (String) : Column name of current position
        value (String) : Value of current cell
        csvLine (Int) : Current line

    Returns:
        None

    Authors:
        Nicolas Atrax
        Marie FU
    """

    if name in [x.lower() for x in TSV_DATE_COLUMNS]:
        checkDate(name, value, csvLine)
    elif value == "" or value == "\n":
        STATE.fileData += "1\t"
    else:
        STATE.fileData += f"{value}\t"
        
    return


def getContent() -> None:

    """
    Get the columns present in the given file

    Args:
        None

    Returns:
        None

    Authors:
        Marie FU
    """
    
    csvLine = 2

    if "zip" in STATE:
        with open(STATE.currentFile, "r", encoding=STATE.encoding) as f:
            reader = csv.DictReader(f, delimiter=STATE.separator)
            for row in reader:
                for name, value in row.items():
                    if name in STATE.columnMap.keys() and value is not None:
                        checkMissing(lowerName(name), value, csvLine)
                csvLine += 1
                STATE.fileData = STATE.fileData[:-1] + "\n"
            STATE.fileData = STATE.fileData[:-1]
    else:
        reader = csv.DictReader(codecs.iterdecode(STATE.file, STATE.encoding), delimiter=STATE.separator)
        for row in reader:
            for name, value in row.items():
                if name in STATE.columnMap.keys() and value is not None:
                    checkMissing(lowerName(name), value, csvLine)
            csvLine += 1
            STATE.fileData = STATE.fileData[:-1] + "\n"
        STATE.fileData = STATE.fileData[:-1]

    return


# Code Start

# page description
st.write(st.session_state.general_text_dict['description'])

# input for a new file name
STATE.newFileName = st.text_input(label=st.session_state.general_text_dict['file_name_input'],placeholder="File.tsv")

# file uploader
st.file_uploader(st.session_state.general_text_dict['file'], type=["tsv", "csv","zip"], key="file")

# checking if a file is uploaded
if STATE.file is not None:

    # make sure to be at start of file before any operation
    # file pointer will be reset each time after each file operation
    toStartOfFile(STATE.file)

    # first utilisation of errMessageLog, contain the log of every error message encountered
    STATE.errMessageLog = ""

    # check new file name given by user
    STATE.newFileName = checkNewFileName(STATE.newFileName, STATE.file.name)
    if STATE.newFileName is None:
        STATE.errMessageLog += st.session_state.general_text_dict['err_file_name']
        errDisplay(STATE.errMessageLog)
    else:
        if STATE.file.type == 'text/tab-separated-values' or STATE.file.type == 'text/csv':
            if "zip" in STATE.keys():
                del STATE.zip

            # check file encoding
            STATE.encoding = checkEncoding(STATE.file)
            toStartOfFile(STATE.file)
            if STATE.encoding is None:
                STATE.errMessageLog += st.session_state.general_text_dict['err_file_encoding']
                errDisplay(STATE.errMessageLog)
            else:
                # get csv separator
                STATE.separator = getSeparator(STATE.file, STATE.encoding)
                toStartOfFile(STATE.file)
                # continue only if separator was found and correct
                if STATE.separator is None:
                    STATE.errMessageLog += STATE.general_text_dict['err_column_separator']
                    errDisplay(STATE.errMessageLog)
                else :
                    # get the mapping of the files columns name to the right columns name
                    STATE.columnMap = {}
                    STATE.columnMap = getColumnsNames(STATE.file, STATE.encoding, STATE.separator, TSV_COLUMNS)
                    if STATE.columnMap != {} and STATE.columnMap != {'choisir une option': 'Abstract'}:

                        toStartOfFile(STATE.file)

                        STATE.fileData = addColumnsNamestoTSV(STATE.file, STATE.encoding, STATE.separator,STATE.columnMap)

                        toStartOfFile(STATE.file)

                        # set a variable to check if there are any detected errors
                        # and construction of correct file content while checking
                        STATE.contentProblem = False
                        getContent()

                        toStartOfFile(STATE.file)
                    
                        if not STATE.contentProblem:
                            STATE.content = ""
                            STATE.content = STATE.file.read().decode('utf-8')
                            if STATE.content == STATE.fileData :
                                st.write(STATE.general_text_dict['correct_file'])
                            else:
                                show_download_button(STATE.newFileName, STATE.fileData.replace("\"","”"))
                        else:
                            errDisplay(STATE.errMessageLog)
        else:
            STATE.zip = True
            STATE.allFileData = ""
            errors = ""

            # temporary directory for input files
            STATE.filesDir = tempfile.TemporaryDirectory()

            # remporary directory for output files
            STATE.zipDir = tempfile.TemporaryDirectory()
            os.mkdir(os.path.join(STATE.zipDir.name, "CleanCSV"))

            with ZipFile(STATE.file, 'r') as zip:
                for info in zip.infolist():
                    if info.filename[-4:] == ".csv" or info.filename[-4:] == ".tsv":
                        zip.extract(info, STATE.filesDir.name)

            for file in os.listdir(STATE.filesDir.name):
                STATE.currentFile = STATE.filesDir.name + "/" + file
                STATE.fileData = "Publication Year\tPublication Month\tPublication Day\tTitle\tAuthors\tSource\tAbstract\n"

                # check file encoding
                STATE.encoding = checkEncoding(STATE.currentFile)
                if STATE.encoding is None:
                    errors += file + ": " + st.session_state.general_text_dict['err_file_encoding'] + "\n"
                else:
                    # get csv separator
                    STATE.separator = getSeparator(STATE.currentFile, STATE.encoding)
                    # continue only if separator was found and correct
                    if STATE.separator is None:
                        errors += file + ": " + STATE.general_text_dict['err_column_separator'] + "\n"
                    else :
                        # get the mapping of the files columns name to the right columns name
                        STATE.columnMap = {}
                        STATE.columnMap = getColumnsNames(STATE.currentFile, STATE.encoding, STATE.separator, TSV_COLUMNS)
                        if STATE.columnMap == {}:
                            errors += file + ": " + STATE.general_text_dict['err_columns'] + "\n"
                        else:

                            STATE.fileData = addColumnsNamestoTSV(STATE.currentFile, STATE.encoding, STATE.separator,STATE.columnMap)

                            # set a variable to check if there are any detected errors
                            # and construction of correct file content while checking
                            STATE.contentProblem = False
                            getContent()
                        
                            if STATE.contentProblem:
                                if STATE.errMessageLog != "":
                                    errors += file + ": \n" + STATE.errMessageLog.replace("\n\n","\n") + "\n"
                            else:
                                with open(STATE.zipDir.name+"/CleanCSV/"+file[:-4]+".tsv","w") as file:
                                    file.write(STATE.fileData.replace("\"","”"))
                                if STATE.allFileData == "":
                                    STATE.allFileData = STATE.fileData
                                else:
                                    STATE.allFileData += STATE.fileData[81:]

            with open(STATE.zipDir.name+"/CleanCSV/compiledFiles.tsv","w") as file:
                file.write(STATE.allFileData.replace("\"","”"))
            
            if errors != "":
                with open(STATE.zipDir.name+"/CleanCSV/all_errors.txt","w") as file:
                    file.write(errors)

            shutil.make_archive(STATE.filesDir.name +"/CleanCSV", "zip", STATE.zipDir.name)

            with open(STATE.filesDir.name +"/CleanCSV.zip", "rb") as zip:
                x = zip.read()
            show_download_button(STATE.newFileName[:-3]+"zip", x)

    STATE.file.close()
    